use embedded_graphics::{
    prelude::*,
    primitives::{PrimitiveStyleBuilder, Rectangle},
};
use embedded_layout::prelude::*;

pub struct Paddding<T, C>
where
    T: Drawable<Color = C> + View,
    C: PixelColor,
{
    pub bounds: Rectangle,
    pub color: Option<C>,
    pub widget: T,
}

impl<T, C> Paddding<T, C>
where
    T: Drawable<Color = C> + View,
    C: PixelColor,
{
    pub fn new(top_left: Point, padding: Size, color: Option<C>, widget: T) -> Self {
        let bounds = Rectangle::new(
            top_left,
            Size::new(
                widget.size().width + (2 * padding.width),
                widget.size().height + (2 * padding.height),
            ),
        );

        Self {
            color,
            widget: widget.align_to(&bounds, horizontal::Center, vertical::Center),
            bounds,
        }
    }
}

impl<T, C> View for Paddding<T, C>
where
    T: Drawable<Color = C> + View,
    C: PixelColor,
{
    fn bounds(&self) -> Rectangle {
        self.bounds
    }

    fn size(&self) -> Size {
        self.bounds.size
    }

    fn translate_impl(&mut self, by: Point) {
        self.bounds.translate_impl(by);
        self.widget.translate_impl(by);
    }
}

impl<T, C> Drawable for Paddding<T, C>
where
    T: Drawable<Color = C> + View,
    C: PixelColor,
{
    type Color = C;
    type Output = ();

    fn draw<D>(&self, target: &mut D) -> Result<Self::Output, D::Error>
    where
        D: DrawTarget<Color = Self::Color>,
    {
        if let Some(color) = self.color {
            let style = PrimitiveStyleBuilder::new().fill_color(color).build();
            self.bounds.into_styled(style).draw(target)?;
        }

        self.widget.draw(target)?;

        Ok(())
    }
}
